%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Maximilian Vogler                                               %
% Name: Inner_Loops.m                                                     %
% Date: 03/12/2021                                                        %
% Purpose: Solves the inner loops, i.e. variable markup and entry.        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [KVEC,KFVEC,MUHHVEC,MUHFVEC,MUFFVEC,MUFHVEC,LAMBDAHVEC,LAMBDAFVEC,PHIFVEC,welfare_moments] = Inner_Loops(sigma,theta,F,tau,ALPHA,RT,ZH,ZF,merger_ind,w,wF,Y0,YF0,vMU,BER,S,merger)

tol=1e-2;                                                                  % set tolerance level for A-B loop

%% Set up loops
MH=size(ZH,1);
MF=size(ZF,1);

MCH=w./ZH;
MCF=wF./ZF;

F = F*476/S;                                                               % This is required to adjust the fixed costs for the simulation size. This is because we have chosen the normalization sum(alpha)=1.  
MCHM=[MCH; tau*MCF];                                                       % This implies that the size of the sector changes as the average alpha decreases as the number of sectors S increases. 
MCFM=[MCF; tau*MCH];                                                       % To keep the model stable, we thus need to adjust the fixed cost F accordingly. Equivalently, we could have had sum(alpha)=S, since this would have implied a constant average alpha of 1.

[MCHM,permH]=sort(MCHM,1);
[MCFM,permF]=sort(MCFM,1);                                                 % sort marginal costs in home and foreign
IOTAH=(permH>MH);                                                          % indicator whether firm is of foreign origin in home market
IOTAF=(permF>MF);                                                          % indicator whether firm is of home origin in foreign market

MFMH=size(MCHM,1); 

if merger==1
% Identify merging firms
    home_firms = (1-IOTAH).*repmat(merger_ind,MFMH,1);                     % these are the home firms in the home market 
    merger_indicator_H = home_firms.*(cumsum(home_firms)<3);               % these are the two largest home firms in the home market

    home_firms = IOTAF.*repmat(merger_ind,MFMH,1);                         % these are the home firms in the foreign market
    merger_indicator_F = home_firms.*(cumsum(home_firms)<3);               % these are the two largest home firms in the foreign market
end

%% Constant Markup

% Set home markups/shares/entry in CMU case
MKP0H=sigma/(sigma-1);                                                     % markup
PHM=(MKP0H*MCHM).^(1-sigma);                                               % p_{i,z}^(1-sigma)                                            
SLASTH=PHM./cumsum(PHM);                                                   % potential s_{i,z} (before entry decision)
checkmatH=(SLASTH>sigma/Y0*w*F./(repmat(ALPHA,MFMH,1)));                   % indicator whether firm (i,z) enters in CMU case               
SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));                % actual s_{i,z} (after entry decision)             
EPSH=sigma;                                                                % used to compute markups at the end of the loop

% Same as above in foreign market
MKP0F=sigma/(sigma-1);
PFM=(MKP0F*MCFM).^(1-sigma);
SLASTF=PFM./cumsum(PFM);     
checkmatF=(SLASTF>sigma/YF0*wF*F./(repmat(ALPHA,MFMH,1)));                 
SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1));                  
EPSF=sigma;

%% Variable Markup 

% Enter loop in VMU case
if vMU==1
    EPSH=vMU_markup(sigma,SHM,BER);                                        % compute variable markups with CMU market shares
    EPSH(EPSH<=1.25)=1.25;                                                 % cap markup EPSH/(EPSH-1) at 5                  
    
    EPSF=vMU_markup(sigma,SFM,BER);
    EPSF(EPSF<=1.25)=1.25; 
    
    iter=0;
    diff=1;
    while (diff>tol) && (iter<101)
        iter=iter+1;
        
        % Update shares in home market
        MKPNH=1/2*(MKP0H+EPSH./(EPSH-1));                                  % set markups for current step
        PHM=(MKPNH.*MCHM).^(1-sigma);                                      % set prices given markups             
        SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));        % calculate shares given prices
        
        if merger==1
            inter=sum(SHM.*merger_indicator_H);                            % combined market share of top two firms
            SHM=inter.*merger_indicator_H+SHM.*(1-merger_indicator_H);     % set market shares of two top firms equal to the sum of them (since this is how they influence the markup calculation)
        end
        
        EPSH=vMU_markup(sigma,SHM,BER);                                    % recalculate markups given prices 
        EPSH(EPSH<=1.25)=1.25;                                             % cap markups at 5 for next step
        
        % Same in foreign market       
        MKPNF=1/2*(MKP0F+EPSF./(EPSF-1));
        PFM=(MKPNF.*MCFM).^(1-sigma);                                       
        SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1));  
        
        if merger==1
            inter=sum(SFM.*merger_indicator_F);
            SFM=inter.*merger_indicator_F+SFM.*(1-merger_indicator_F);
        end
        
        EPSF=vMU_markup(sigma,SFM,BER);
        EPSF(EPSF<=1.25)=1.25;
        
        % Check for convergence
        diff=sum(sum(abs([MKPNH-MKP0H;MKPNF-MKP0F])));
        
        % Update markups
        MKP0H=MKPNH;
        MKP0F=MKPNF;

    end
    
    % Check entry conditions again with variable markups
    PHM=(MKP0H.*MCHM).^(1-sigma);                                           
    SLASTH=PHM./cumsum(PHM);
    checkmatH=(SLASTH>sigma/Y0*w*F./(repmat(ALPHA,MFMH,1)));               % determine which firms enter with CMU fringe - this is the crucial new element
    SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));
    
    if merger==1
        inter=sum(SHM.*merger_indicator_H);                                
        SHM=inter.*merger_indicator_H+SHM.*(1-merger_indicator_H);
    end

    EPSH=vMU_markup(sigma,SHM,BER);
    EPSH(EPSH<=1.25)=1.25; 
    
    % Same in foreign market
    PFM=(MKP0F.*MCFM).^(1-sigma);                                          
    SLASTF=PFM./cumsum(PFM);     
    checkmatF=(SLASTF>sigma/YF0*wF*F./(repmat(ALPHA,MFMH,1)));              
    SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1)); 
    
    if merger==1
        inter=sum(SFM.*merger_indicator_F);
        SFM=inter.*merger_indicator_F+SFM.*(1-merger_indicator_F);
    end
    
    EPSF=vMU_markup(sigma,SFM,BER);
    EPSF(EPSF<=1.25)=1.25;
    
    % Same loop as before, but with new entrants
    iter=0;
    diff=1;
    while (diff>tol) && (iter<101)
        iter=iter+1;

        MKPNH=1/2*(MKP0H+EPSH./(EPSH-1));
        PHM=(MKPNH.*MCHM).^(1-sigma);                                     
        SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));
        
        if merger==1
            inter=sum(SHM.*merger_indicator_H);                                
            SHM=inter.*merger_indicator_H+SHM.*(1-merger_indicator_H);
        end
        
        EPSH=vMU_markup(sigma,SHM,BER);
        EPSH(EPSH<=1.25)=1.25;
        
        % Same in foreign market
        MKPNF=1/2*(MKP0F+EPSF./(EPSF-1));
        PFM=(MKPNF.*MCFM).^(1-sigma);                                           
        SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1));   
        
        if merger==1
            inter=sum(SFM.*merger_indicator_F);
            SFM=inter.*merger_indicator_F+SFM.*(1-merger_indicator_F);
        end
        
        EPSF=vMU_markup(sigma,SFM,BER);
        EPSF(EPSF<=1.25)=1.25;

        % Check for convergence
        diff=sum(sum(abs([MKPNH-MKP0H;MKPNF-MKP0F])));

        % Update markups
        MKP0H=MKPNH;
        MKP0F=MKPNF;

    end
end

if merger==1
    % Restore true market shares
    SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));
    SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1)); 
end 

checkmatH=(SHM>0);                                                         % matrix of firms that enter in home in equilibrium above
checkmatF=(SFM>0); 
MUHM=EPSH./(EPSH-1);                                                       % markups of firms in home
MUFM=EPSF./(EPSF-1);

%% Compute Summary Statistics 

KVEC=sum(checkmatH);                                                       % vector of number of firms that enter in home (z,1)
KFVEC=sum(checkmatF);                                                      % vector of number of firms that enter in foreign (z,1)
PHIFVEC=1./(1+(tau*w/wF)^theta./RT);                                       % expected import share vector in foregin PHI*
MUHHVEC=sum((1-IOTAH).*SHM./MUHM);                                         
MUHFVEC=sum(IOTAF.*SFM./MUFM);
MUFFVEC=sum((1-IOTAF).*SFM./MUFM);
MUFHVEC=sum(IOTAH.*SHM./MUHM);
LAMBDAHVEC=sum(IOTAH.*SHM);                                                % realized import share vector in home LAMBDA
LAMBDAFVEC=sum(IOTAF.*SFM);                                                % realized import share vector in foreign LAMBDA* 

% Auxiliary variables 
KHH = sum(checkmatH.*(1-IOTAH));                                           % number of home firms active in home for each sector
KHF = sum(checkmatF.*IOTAF);                                               % number of home firms active in foreign for each sector

% Welfare moments
P_index = sum(PHM.*checkmatH).^(1/(1-sigma));                              % home price index
PF_index = sum(PFM.*checkmatF).^(1/(1-sigma));                             % foreign price index
profits = sum(SHM.*(1-IOTAH)./EPSH).*ALPHA*Y0 + ...                        % total profits of home firms (in both home and foreign markets) 
          sum(SFM.*IOTAF./EPSF).*ALPHA*YF0 - ...
          KHH*w*F-KHF*wF*F;    
welfare = profits./(ALPHA*Y0)-log(P_index);                                % total welfare in each sector

welfare_moments = zeros(4,length(welfare));
welfare_moments(1,:) = P_index;
welfare_moments(2,:) = profits;
welfare_moments(3,:) = welfare;
welfare_moments(4,:) = PF_index;

end