%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Maximilian Vogler                                               %
% Name: Main_Merger.m                                                     %
% Date: 03/12/2021                                                        %
% Purpose: This code computes the merger simulation in PE and GE,         %
% computes welfare effects and saves the plotting data for Fig1(a),       %
% Fig1(b), Fig5(a) and Fig5(b).                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Housekeeping
clear;
close all;
clc

tstart0 = tic;

addpath('Data');
addpath('Auxiliary_Functions');
addpath('Results');
set(0,'DefaultLegendAutoUpdate','off')

%% Load Parameters 
load('Baseline_Parameters')                                                
load('GE_Results','Y','YF')

% Extract parameters
muT = bestParams(1,1);
sigmaT = bestParams(1,2);
tau = bestParams(1,3);
kappa = bestParams(1,4);
f = bestParams(1,5);

% Fix sigma
sigma = 5;

% Compute derivative parameters
theta = (sigma-1)*kappa;
F = f*4.93*.43*10^(-5)/5;

% Wages (with domestic wage as numeraire and calibrated relative wage)
wR = 1.13;      
w = 1;          
wF = 1/wR; 

% Labor Normalization
L0 = 100;

%% Merger specific variables

IP = 10;                                                                   % Number of bins in which policy changed
sortG = 1;                                                                 % What to sort bins by (==1 -> sort by Gamma, ==0 --> sort by LambdaF) 
rho_vec = [0.1,0.35,0.5];                                                  % Vector of spillovers
rlength = length(rho_vec);

%% Setup of model 

% Initialize random number generator and set seed
aseed = 1;
rng(aseed);

% Number of shadow firms
MF_large=10000;
MF_small=1400;
MH_large=5000;
MH_small=700;

% Markups
vMU = 1;                                                                   % = 1 for variable markups

% Competition Structure
BER = 0;                                                                   % = 0 for Cournot competition

% Compute number of sectors 
cdshares_init = csvread('cdshares_v3.csv');                                % Cobb-Douglas shares in data

S_multiple = 4;                                                            % Multiplication factor for sectors
S_init = length(cdshares_init);                                            % Number of sectors in data
S = S_init*S_multiple;                                                     % Number of sectors used

% Assign CD-shares across sectors 
ALPHA = zeros(S,1);
ALPHA(1:S_init,1) = cdshares_init;                                         % Same as in the data
for i = 1:S_multiple - 1                                                   % Random permutations for additional sectors
    ALPHA(S_init*i+1:S_init*(i+1),1) = cdshares_init(randperm(S_init));
end
ALPHA = ALPHA/S_multiple;

% Split sectors into large and small based on CD-share.
split_param = 1.25;
small = (ALPHA<split_param/S);                            
Nsmall = sum(small);                           
Nlarge = S-Nsmall;

%% Productivity draws

rng(aseed);                                              
rtdraws = randn(1,S);

% Draw fundamental Productivites T_z for each z 
RT=exp(muT+sigmaT*rtdraws);
RTS=RT(small);
RTL=RT(~small);

% Draw idiosyncratic productivities varphi_{i,z} for each z and i
% Small firm U_{i,z}
UH0S = exprnd(1,MH_small,Nsmall);                                          % Draw U of most productive small home shadow firm and spacings in each sector from exponential with mean 1
UF0S = exprnd(1,MF_small,Nsmall);                        
UHS = cumsum(UH0S);                                                        % Cumulate to get U of each home firm in each sector
UFS = cumsum(UF0S);

% Large firm U_{i,z}
UH0L = exprnd(1,MH_large,Nlarge);
UF0L = exprnd(1,MF_large,Nlarge);
UHL = cumsum(UH0L);
UFL = cumsum(UF0L);

% Transform U_{i,z} into varphi_{i,z}   
ZHS=(UHS./(repmat(RTS,MH_small,1))).^(-1/theta);
ZFS=UFS.^(-1/theta);
ZHL=(UHL./(repmat(RTL,MH_large,1))).^(-1/theta);                        
ZFL=UFL.^(-1/theta);

% Set up matrices for all information that needs to be saved
Y_baseline = zeros(rlength,1);
YF_baseline = zeros(rlength,1);
baseline_welfare = zeros(rlength,1);
baseline_indices = zeros(rlength,1);
baseline_indiceF = zeros(rlength,1);
baseline_profits = zeros(rlength,1);
baseline_welfarF = zeros(rlength,1);
PE_merger_welfare = zeros(rlength,IP);
PE_merger_indices = zeros(rlength,IP);
PE_merger_profits = zeros(rlength,IP);
PE_merger_indiceF = zeros(rlength,IP);
PE_merger_welfarF = zeros(rlength,IP);
GE_merger_welfare = zeros(rlength,IP);
GE_merger_indices = zeros(rlength,IP);
GE_merger_profits = zeros(rlength,IP);
GE_merger_indiceF = zeros(rlength,IP);
GE_merger_welfarF = zeros(rlength,IP);

baseline_local_welfare = zeros(S,rlength);
baseline_local_indices = zeros(S,rlength);
baseline_local_profits = zeros(S,rlength);
baseline_local_indiceF = zeros(S,rlength);
PEmerger_local_welfare = zeros(S,rlength,IP);
PEmerger_local_indices = zeros(S,rlength,IP);
PEmerger_local_profits = zeros(S,rlength,IP);
PEmerger_local_indiceF = zeros(S,rlength,IP);
GEmerger_local_welfare = zeros(S,rlength,IP);
GEmerger_local_indices = zeros(S,rlength,IP);
GEmerger_local_profits = zeros(S,rlength,IP);
GEmerger_local_indiceF = zeros(S,rlength,IP);

GammaF = zeros(S,rlength);
bin_weight = zeros(rlength,IP);

bar_object_home = zeros(IP,rlength);
bar_object_foreign = zeros(IP,rlength);
plot_object = zeros(4,rlength,IP);
count = 1;

%% Compute outcomes for different levels of spillovers
for i = 1:rlength
    
    spillover = rho_vec(i);
    
    %% Compute GE baseline

    % Guess home and foreign output Y and YF
    Y0=Y;
    YF0=YF;

    % Run loops to solve GE model 
    [Y_baseline(i),YF_baseline(i),LF,PHIFVEC,LAMBDAFVEC,baseline_welfare_moments] = GE_merger(sigma,theta,F,tau,ALPHA,RTS,RTL,ZHS,ZFS,ZHL,ZFL,zeros(1,S),zeros(1,S),w,wF,L0,Y0,YF0,small,vMU,BER,0);
    
    disp('Made it past first loop')
    % Compute sectoral moments
    baseline_local_indices(:,i) = baseline_welfare_moments(1,:);
    baseline_local_profits(:,i) = baseline_welfare_moments(2,:);
    baseline_local_welfare(:,i) = baseline_welfare_moments(3,:);
    baseline_local_indiceF(:,i) = baseline_welfare_moments(4,:);
    
    % Compute aggregate welfare
    baseline_indices(i) = exp(ALPHA'*log(baseline_local_indices(:,i)));
    baseline_welfare(i) = Y_baseline(i)/baseline_indices(i);
    baseline_profits(i) = sum(baseline_local_profits(:,i));
    baseline_indiceF(i) = exp(ALPHA'*log(baseline_local_indiceF(:,i)));
    baseline_welfarF(i) = YF_baseline(i)/baseline_indiceF(i);
    
    % Compute granular trade share GammaF
    GammaF(:,i) = (LAMBDAFVEC-PHIFVEC)';
    
    % Compute bin in which each sector is according to GammaF or LambdaF
    if sortG == 1
        [bounds,bin_index] = Binning(GammaF(:,i),IP);
    elseif sortG == 0
        [bounds,bin_index] = Binning(LAMBDAFVEC',IP);
    else
        error('sortG has to be 0 or 1')
    end
    
    
    % Compute outcomes if mergers take place in different sector bins
    for j = 1:IP
        
        % Compute weight of bin
        bin_weight(i,j) = sum(ALPHA(bin_index==j));             
        
        % Identify merging sectors
        merger_indS = bin_index(small)==j;
        merger_indL = bin_index(~small)==j;
        
        % Recompute varphi of the second-biggest firm in merged sectors
        ZHS_merger = ZHS;
        ZHS_merger(2,merger_indS) = spillover * ZHS(1,merger_indS) + (1-spillover) * ZHS(2,merger_indS);
        
        ZHL_merger = ZHL;
        ZHL_merger(2,merger_indL) = spillover * ZHL(1,merger_indL) + (1 - spillover) * ZHL(2,merger_indL);
        
        %% Merger in GE
        
        % Run loops to solve GE model
        [Y_GEmerger,YF_GEmerger,wF,~,~,GEmerger_welfare_moments] = GE_wF_merger(sigma,theta,F,tau,ALPHA,RTS,RTL,ZHS_merger,ZFS,ZHL_merger,ZFL,merger_indS',merger_indL',w,wF,L0,LF,Y0,YF0,small,vMU,BER,1);
 
        disp('Made it past third loop')
        disp('');
        
        % Compute sectoral moments
        GEmerger_local_indices(:,i,j) = GEmerger_welfare_moments(1,:);
        GEmerger_local_profits(:,i,j) = GEmerger_welfare_moments(2,:);
        GEmerger_local_welfare(:,i,j) = GEmerger_welfare_moments(3,:);
        GEmerger_local_indiceF(:,i,j) = GEmerger_welfare_moments(4,:);
        
        % Compute aggregate welfare
        GE_merger_indices(i,j) = exp(ALPHA'*log(GEmerger_local_indices(:,i,j)));
        GE_merger_welfare(i,j) = Y_GEmerger/GE_merger_indices(i,j);
        GE_merger_profits(i,j) = sum(GEmerger_local_profits(:,i,j));
        GE_merger_indiceF(i,j) = exp(ALPHA'*log(GEmerger_local_indiceF(:,i,j)));
        GE_merger_welfarF(i,j) = YF_GEmerger/GE_merger_indiceF(i,j);
        
        disp('----------')
    end
    fprintf('Convergence of rho %d/%d.\n', i, rlength)
    disp('-----------------------------')
    
    bar_object_home(:,i) = ((GE_merger_welfare(i,:)/baseline_welfare(i)-1)./bin_weight(i,:)*100)';
    bar_object_foreign(:,i) = ((GE_merger_welfarF(i,:)/baseline_welfarF(i)-1)./bin_weight(i,:)*100)';
    bar_object = [bar_object_home(:,i),bar_object_foreign(:,i)];
    
    plot_object(1,i,:) = (GE_merger_welfare(i,:)/baseline_welfare(i)-1)./bin_weight(i,:)*100;
    plot_object(2,i,:) = (GE_merger_welfarF(i,:)/baseline_welfarF(i)-1)./bin_weight(i,:)*100;
    plot_object(3,i,:) = ((GE_merger_profits(i,:)-baseline_profits(i))/Y_baseline(i))./bin_weight(i,:)*100;
    plot_object(4,i,:) = -sum(ALPHA.*(log(squeeze(GEmerger_local_indices(:,i,:)))-log(baseline_local_indices(:,i))))./bin_weight(i,:)*100;
    
end

if sortG == 1
    save('Results/Plot_Data/Figure1_Data_Gamma','bar_object_home','bar_object_foreign','plot_object');
else 
    save('Results/Plot_Data/Figure1_Data_Lambda','bar_object_home','bar_object_foreign','plot_object');
end

