%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Maximilian Vogler                                               %
% Name: Main_Merger                                                       %
% Date: 03/19/2021                                                        %
% Purpose: This code computes the merger simulation in PE and GE,         %
% computes welfare effects and saves the plotting data for Fig2(a)(b).    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Housekeeping
clear;
close all;
clc

tstart0 = tic;

addpath('Data');
addpath('Auxiliary_Functions');
addpath('Results');
set(0,'DefaultLegendAutoUpdate','off')

%% Load Parameters 
load('Baseline_Parameters')
load('GE_Results','Y','YF')

% Extract parameters
muT = bestParams(1,1);
sigmaT = bestParams(1,2);
tau = bestParams(1,3);
kappa = bestParams(1,4);
f = bestParams(1,5);

% Fix sigma
sigma = 5;

% Compute derivative parameters
theta = (sigma-1)*kappa;
F = f*4.93*.43*10^(-5)/5;

% Wages (with domestic wage as numeraire and calibrated relative wage)
wR = 1.13;      
w = 1;          
wF = 1/wR; 

% Labor Normalization
L0 = 100;

%% Merger specific variables

tauInd = 0;                                                                % Comparative static (==1 --> tau, ==0 --> rho)

if tauInd == 1
    tau_vec = [1.35, 1.45, 1.55, 1.65, 1.75, 1.85];
    spillover = 0.35;
    tlength = length(tau_vec);
elseif tauInd == 0
    rho_vec = [0.1,0.2,0.3,0.4,0.5,0.6];
    current_tau = tau;
    tlength = length(rho_vec);
else
    error('tauInd has to be 0 or 1')
end
IP = 5;                                                                    % Number of bins in which policy changed

%% Setup of model 

% Initialize random number generator and set seed
aseed = 1;
rng(aseed);

% Number of shadow firms
MF_large=10000;
MF_small=1400;
MH_large=5000;
MH_small=700;

% Markups
vMU = 1;                                                                   % = 1 for variable markups

% Competition Structure
BER = 0;                                                                   % = 1 for Bertrand competition

% Compute number of sectors 
cdshares_init = csvread('cdshares_v3.csv');                                % Cobb-Douglas shares in data

S_multiple = 4;                                                            % Multiplication factor for sectors
S_init = length(cdshares_init);                                            % Number of sectors in data
S = S_init*S_multiple;                                                     % Number of sectors used

% Assign CD-shares across sectors 
ALPHA = zeros(S,1);
ALPHA(1:S_init,1) = cdshares_init;                                         % Same as in the data
for i = 1:S_multiple - 1                                                   % Random permutations for additional sectors
    ALPHA(S_init*i+1:S_init*(i+1),1) = cdshares_init(randperm(S_init));
end
ALPHA = ALPHA/S_multiple;

% Split sectors into large and small based on CD-share.
split_param = 1.25;
small = (ALPHA<split_param/S);                            
Nsmall = sum(small);                           
Nlarge = S-Nsmall;

%% Productivity draws

rng(aseed);                                              
rtdraws = randn(1,S);

% Draw fundamental Productivites T_z for each z 
RT=exp(muT+sigmaT*rtdraws);
RTS=RT(small);
RTL=RT(~small);

% Draw idiosyncratic productivities varphi_{i,z} for each z and i
% Small firm U_{i,z}
UH0S = exprnd(1,MH_small,Nsmall);                                          % Draw U of most productive small home shadow firm and spacings in each sector from exponential with mean 1
UF0S = exprnd(1,MF_small,Nsmall);                        
UHS = cumsum(UH0S);                                                        % Cumulate to get U of each home firm in each sector
UFS = cumsum(UF0S);

% Large firm U_{i,z}
UH0L = exprnd(1,MH_large,Nlarge);
UF0L = exprnd(1,MF_large,Nlarge);
UHL = cumsum(UH0L);
UFL = cumsum(UF0L);

% Transform U_{i,z} into varphi_{i,z}   
ZHS=(UHS./(repmat(RTS,MH_small,1))).^(-1/theta);
ZFS=UFS.^(-1/theta);
ZHL=(UHL./(repmat(RTL,MH_large,1))).^(-1/theta);                        
ZFL=UFL.^(-1/theta);

% Set up matrices for all information that needs to be saved
Y_baseline = zeros(tlength,1);
YF_baseline = zeros(tlength,1);
Y_GEmerger_store = zeros(IP,1);                                            % needed only for debugging
YF_GEmerger_store = zeros(IP,1);                                           % needed only for debugging
baseline_welfare = zeros(tlength,1);
baseline_indices = zeros(tlength,1);
baseline_indiceF = zeros(tlength,1);
baseline_profits = zeros(tlength,1);
baseline_welfarF = zeros(tlength,1);
PE_merger_welfare = zeros(tlength,IP);
PE_merger_indices = zeros(tlength,IP);
PE_merger_profits = zeros(tlength,IP);
PE_merger_indiceF = zeros(tlength,IP);
PE_merger_welfarF = zeros(tlength,IP);
GE_merger_welfare = zeros(tlength,IP);
GE_merger_indices = zeros(tlength,IP);
GE_merger_profits = zeros(tlength,IP);
GE_merger_indiceF = zeros(tlength,IP);
GE_merger_welfarF = zeros(tlength,IP);

baseline_local_welfare = zeros(S,tlength);
baseline_local_indices = zeros(S,tlength);
baseline_local_profits = zeros(S,tlength);
baseline_local_indiceF = zeros(S,tlength);
PEmerger_local_welfare = zeros(S,tlength,IP);
PEmerger_local_indices = zeros(S,tlength,IP);
PEmerger_local_profits = zeros(S,tlength,IP);
PEmerger_local_indiceF = zeros(S,tlength,IP);
GEmerger_local_welfare = zeros(S,tlength,IP);
GEmerger_local_indices = zeros(S,tlength,IP);
GEmerger_local_profits = zeros(S,tlength,IP);
GEmerger_local_indiceF = zeros(S,tlength,IP);

GammaF = zeros(S,tlength);
bin_weight = zeros(tlength,IP);

welfare = zeros(tlength,2);

% Compute outcomes for different levels of trade openness
for i = 1:tlength
    
    if tauInd == 1
        current_tau = tau_vec(i);
    else
        spillover = rho_vec(i);
    end
    
    %% Compute GE baseline

    % Guess home and foreign output Y and YF
    Y0=Y;
    YF0=YF;

    % Run loops to solve GE model 
    [Y_baseline(i),YF_baseline(i),LF,PHIFVEC,LAMBDAFVEC,baseline_welfare_moments] = GE_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS,ZFS,ZHL,ZFL,zeros(1,S),zeros(1,S),w,wF,L0,Y0,YF0,small,vMU,BER,0);
    
    disp('Made it past first loop')
    % Compute sectoral moments
    baseline_local_indices(:,i) = baseline_welfare_moments(1,:);
    baseline_local_profits(:,i) = baseline_welfare_moments(2,:);
    baseline_local_welfare(:,i) = baseline_welfare_moments(3,:);
    baseline_local_indiceF(:,i) = baseline_welfare_moments(4,:);
    
    % Compute aggregate welfare
    baseline_indices(i) = exp(ALPHA'*log(baseline_local_indices(:,i)));
    baseline_welfare(i) = Y_baseline(i)/baseline_indices(i);
    baseline_profits(i) = sum(baseline_local_profits(:,i));
    baseline_indiceF(i) = exp(ALPHA'*log(baseline_local_indiceF(:,i)));
    baseline_welfarF(i) = YF_baseline(i)/baseline_indiceF(i);
    
    % Compute granular trade share Gamma*
    GammaF(:,i) = (LAMBDAFVEC-PHIFVEC)';
    
    % Compute bin in which each sector is according to Gamma*
    [bounds,bin_index] = Binning(GammaF(:,i),IP);
%     [bounds,bin_index] = Binning(LAMBDAFVEC',IP);
    
    
    % Compute outcomes if mergers take place in different sector bins
    j = IP;
        
    % Compute weight of bin
    bin_weight(i,j) = sum(ALPHA(bin_index==j));             

    % Identify merging sectors
    merger_indS = bin_index(small)==j;
    merger_indL = bin_index(~small)==j;

    % Recompute varphi of the second-biggest firm in merged sectors
    ZHS_merger = ZHS;
    ZHS_merger(2,merger_indS) = spillover * ZHS(1,merger_indS) + (1-spillover) * ZHS(2,merger_indS);

    ZHL_merger = ZHL;
    ZHL_merger(2,merger_indL) = spillover * ZHL(1,merger_indL) + (1 - spillover) * ZHL(2,merger_indL);

    %% Merger in GE

    % Run loops to solve GE model
    [Y_GEmerger,YF_GEmerger,wF,~,~,GEmerger_welfare_moments] = GE_wF_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS_merger,ZFS,ZHL_merger,ZFL,merger_indS',merger_indL',w,wF,L0,LF,Y0,YF0,small,vMU,BER,1);
    disp('Made it past third loop')
    disp('');
    % Compute sectoral moments
    GEmerger_local_indices(:,i,j) = GEmerger_welfare_moments(1,:);
    GEmerger_local_profits(:,i,j) = GEmerger_welfare_moments(2,:);
    GEmerger_local_welfare(:,i,j) = GEmerger_welfare_moments(3,:);
    GEmerger_local_indiceF(:,i,j) = GEmerger_welfare_moments(4,:);

    % Compute aggregate welfare
    GE_merger_indices(i,j) = exp(ALPHA'*log(GEmerger_local_indices(:,i,j)));
    GE_merger_welfare(i,j) = Y_GEmerger/GE_merger_indices(i,j);
    GE_merger_profits(i,j) = sum(GEmerger_local_profits(:,i,j));
    GE_merger_indiceF(i,j) = exp(ALPHA'*log(GEmerger_local_indiceF(:,i,j)));
    GE_merger_welfarF(i,j) = YF_GEmerger/GE_merger_indiceF(i,j);

    disp('----------')
    if tauInd == 1
        fprintf('Convergence of tau %d/%d.\n', i, tlength)
    else
        fprintf('Convergence of rho %d/%d.\n', i, tlength)
    end
    disp('-----------------------------')
    
    welfare(i,1) = (GE_merger_welfare(i,IP)/baseline_welfare(i)-1)./bin_weight(i,IP)*100; % Home welfare
    welfare(i,2) = (GE_merger_welfarF(i,IP)/baseline_welfarF(i)-1)./bin_weight(i,IP)*100; % Foreign welfare
     
end

if tauInd == 1
    save('Results/Plot_Data/Figure2_Data_tau','tau_vec','welfare');
else
    save('Results/Plot_Data/Figure2_Data_rho','rho_vec','welfare');
end

