%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Maximilian Vogler                                               %
% Name: Fig3_Data.m                                                       %
% Date: 03/26/2021                                                        %
% Purpose: Computes data for Figure 3                                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Housekeeping
clear;
close all;
clc

tstart0 = tic;

addpath('Data');
addpath('Auxiliary_Functions');
addpath('Results');

%% Load Parameters 
load('Baseline_Parameters') 
load('GE_Results','Y','YF')

% Extract parameters
muT = bestParams(1,1);
sigmaT = bestParams(1,2);
tau = bestParams(1,3);
kappa = bestParams(1,4);
f = bestParams(1,5);

% Fix sigma
sigma = 5;

% Compute derivative parameters
theta = (sigma-1)*kappa;
F = f*4.93*.43*10^(-5)/5;

% Wages (with domestic wage as numeraire and calibrated relative wage)
wR = 1.13;      
w = 1;          
wF = 1/wR; 

wF_save = wF;

% Labor Normalization
L0 = 100;

%% Merger specific variables

tau_vec = linspace(1,2,10);                                                % Measure degree of trade openness                                                      % Measures strength of spillover
spillover = 0.35;                                                           % Measure spillovers
IP = 5;                                                                    % Number of bins in which policy changed
lambda0 = 1;

lambda_length = 3;
tau_length = length(tau_vec);
    
%% Setup of model 

% Initialize random number generator and set seed
aseed = 1;
rng(aseed);

% Number of shadow firms
MF_large=10000;
MF_small=1400;
MH_large=5000;
MH_small=700;

% Markups
vMU = 1;                                                                   % = 1 for variable markups

% Competition Structure
BER = 0;                                                                   % = 1 for Bertrand competition

% Compute number of sectors 
cdshares_init = csvread('cdshares_v3.csv');                                % Cobb-Douglas shares in data

S_multiple = 4;                                         
S_init = length(cdshares_init);                                            % Number of sectors in data
S = S_init*S_multiple;                                                     % Number of sectors used

% Assign CD-shares across sectors 
ALPHA = zeros(S,1);
ALPHA(1:S_init,1) = cdshares_init;                                         % Same as in the data
for i = 1:S_multiple - 1                                                   % Random permutations for additional sectors
    ALPHA(S_init*i+1:S_init*(i+1),1) = cdshares_init(randperm(S_init));
end
ALPHA = ALPHA/S_multiple;

% Split sectors into large and small based on CD-share.
split_param = 1.25;
small = (ALPHA<split_param/S);                            
Nsmall = sum(small);                           
Nlarge = S-Nsmall;

%% Productivity draws

rng(aseed);                                              
rtdraws = randn(1,S);

% Draw fundamental Productivites T_z for each z 
RT=exp(muT+sigmaT*rtdraws);
RTS=RT(small);
RTL=RT(~small);

% Draw idiosyncratic productivities varphi_{i,z} for each z and i
% Small firm U_{i,z}
UH0S = exprnd(1,MH_small,Nsmall);                                          % Draw U of most productive small home shadow firm and spacings in each sector from exponential with mean 1
UF0S = exprnd(1,MF_small,Nsmall);                        
UHS = cumsum(UH0S);                                                        % Cumulate to get U of each home firm in each sector 
UFS = cumsum(UF0S);

% Large firm U_{i,z}
UH0L = exprnd(1,MH_large,Nlarge);
UF0L = exprnd(1,MF_large,Nlarge);
UHL = cumsum(UH0L);
UFL = cumsum(UF0L);

% Transform U_{i,z} into varphi_{i,z}   
ZHS=(UHS./(repmat(RTS,MH_small,1))).^(-1/theta);
ZFS=UFS.^(-1/theta);
ZHL=(UHL./(repmat(RTL,MH_large,1))).^(-1/theta);                        
ZFL=UFL.^(-1/theta);

% Set up matrices for all information that needs to be saved
merger_local_indices = zeros(S,tau_length);
merger_local_profits = zeros(S,tau_length);
merger_local_welfare = zeros(S,tau_length);
merger_local_welfareF = zeros(S,tau_length);

merger_indices = zeros(tau_length,1);
merger_profits = zeros(tau_length,1);
merger_welfare = zeros(tau_length,1);
merger_share = zeros(tau_length,1);

exports = zeros(tau_length);

%% Compute optimal mergers for different levels of lambda and tau
for l = 1:lambda_length
    
    wF = wF_save;
    
    for i = 1:tau_length

        % Set spillover for this run
        current_tau = tau_vec(i);
        iteration = 0;
        merger_indS_old = zeros(Nsmall,1);
        merger_indL_old = zeros(Nlarge,1);

        %% Compute GE baseline

        % Guess home and foreign output Y and YF
        Y0=Y;
        YF0=YF;

        % Run loops to solve GE model 
        [Y_baseline,YF_baseline,LF,PHIFVEC,LAMBDAFVEC,baseline_welfare_moments] = GE_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS,ZFS,ZHL,ZFL,zeros(1,S),zeros(1,S),w,wF,L0,Y0,YF0,small,vMU,BER,0);

        % Determine lambda
        if l == 1
            lambda = 0;
        elseif l == 2
            lambda = LF/L0;
        elseif l == 3
            lambda = 1;
            lambda0 = 0;
        else
            error('Not enough lambda values')
        end
        
        % Compute sectoral moments
        baseline_local_indices = baseline_welfare_moments(1,:);
        baseline_local_profits = baseline_welfare_moments(2,:);
        baseline_local_welfare = baseline_welfare_moments(3,:);
        baseline_local_welfareF = baseline_welfare_moments(4,:);

        % Compute aggregate welfare
        baseline_indices = exp(ALPHA'*log(baseline_local_indices(:)));
        baseline_welfare = Y_baseline/baseline_indices;
        baseline_profits = sum(baseline_local_profits(:));

        Y0 = Y_baseline;
        YF0 = YF_baseline;
        fprintf('Initial GE in tau %d/%d of lambda %d/%d computed.\n',i, tau_length, l, lambda_length)

        while true

            % First PE Run: No mergers                                              
            merger_indS = zeros(Nsmall,1);
            merger_indL = zeros(Nlarge,1);

            % Run PE Loop
            [~,~,~,~,~,~,~,~,~,~,PE_no_merger_welfare_moments] = PE_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS,ZFS,ZHL,ZFL,merger_indS',merger_indL',w,wF,Y0,YF0,small,vMU,BER,0);

            PE_no_merger_local_welfare = PE_no_merger_welfare_moments(3,:); 
            PE_no_merger_local_welfareF = PE_no_merger_welfare_moments(4,:);
            PE_no_merger_local_combined = lambda0 * PE_no_merger_local_welfare + lambda * PE_no_merger_local_welfareF;

            % Second PE Run: All merger
            merger_indS = ones(Nsmall,1);                                           
            merger_indL = ones(Nlarge,1);

            % Recompute varphi of the second-biggest firm in merged sectors
            ZHS_merger = ZHS;
            ZHS_merger(2,:) = spillover * ZHS(1,:) + (1-spillover) * ZHS(2,:);

            ZHL_merger = ZHL;
            ZHL_merger(2,:) = spillover * ZHL(1,:) + (1 - spillover) * ZHL(2,:);

            [~,~,~,~,~,~,~,~,~,~,PE_all_merger_welfare_moments] = PE_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS_merger,ZFS,ZHL_merger,ZFL,merger_indS',merger_indL',w,wF,Y0,YF0,small,vMU,BER,1);

            PE_all_merger_local_welfare = PE_all_merger_welfare_moments(3,:);
            PE_all_merger_local_welfareF = PE_all_merger_welfare_moments(4,:);
            PE_all_merger_local_combined = lambda0 * PE_all_merger_local_welfare + lambda * PE_all_merger_local_welfareF;

            % Find merging sectors given GE
            merger_indS_GE = (PE_all_merger_local_combined(small) > PE_no_merger_local_combined(small))';
            merger_indL_GE = (PE_all_merger_local_combined(~small) > PE_no_merger_local_combined(~small))';

            % Recompute varphi of the second-biggest firm in merged sectors
            ZHS_merger = ZHS;
            ZHS_merger(2,merger_indS_GE) = spillover * ZHS(1,merger_indS_GE) + (1-spillover) * ZHS(2,merger_indS_GE);

            ZHL_merger = ZHL;
            ZHL_merger(2,merger_indL_GE) = spillover * ZHL(1,merger_indL_GE) + (1 - spillover) * ZHL(2,merger_indL_GE);

            % Check convergence (and count number of divergences)
            difference = 0;
            for j = 1:Nsmall
                if merger_indS_GE(j,1) ~= merger_indS_old(j,1)
                    difference = difference + 1;
                end
            end
            for j = 1:Nlarge
                if merger_indL_GE(j,1) ~= merger_indL_old(j,1)
                    difference = difference + 1;                                
                end
            end
            fprintf('The difference in iteration %d of tau %d/%d in lambda %d/%d is %d.\n', iteration, i, tau_length, l, lambda_length, difference)
            if (difference == 0) || (iteration > 5)
                [~,~,~,~,~,~,~,~,~,LAMBDAFVEC,PE_merger_welfare_moments] = PE_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS_merger,ZFS,ZHL_merger,ZFL,merger_indS_GE',merger_indL_GE',w,wF,Y0,YF0,small,vMU,BER,1);
                exports(i) = mean(LAMBDAFVEC);
                merger_local_indices(:,i) = PE_merger_welfare_moments(1,:);    
                merger_local_profits(:,i) = PE_merger_welfare_moments(2,:);
                merger_local_welfare(:,i) = PE_merger_welfare_moments(3,:); 
                merger_local_welfareF(:,i) = PE_merger_welfare_moments(4,:);

                % Compute aggregate welfare
                merger_indices(i) = exp(ALPHA'*log(merger_local_indices(:,i)));
                merger_welfare(i) = Y0/merger_indices(i);
                merger_profits(i) = sum(merger_local_profits(:,i));
                merger_share(i) = (sum(merger_indS_GE) + sum(merger_indL_GE))/S;
                fprintf('Convergence of tau %d/%d in lambda %d/%d in %d iterations.\n', i, tau_length, l, lambda_length, iteration)
                disp('-----------------------------')
                break;
            end

            % Run GE Loop
            [Y0,YF0,wF,~,~,GE_merger_welfare_moments] = GE_wF_merger(sigma,theta,F,current_tau,ALPHA,RTS,RTL,ZHS_merger,ZFS,ZHL_merger,ZFL,merger_indS_GE',merger_indL_GE',w,wF,L0,LF,Y0,YF0,small,vMU,BER,1);

            merger_indS_old = merger_indS_GE;
            merger_indL_old = merger_indL_GE;

            iteration = iteration + 1;

        end

    end
    
    if l == 1
        lambda = 0;
        save('Results/Data/Fig3_0','tau_vec','merger_share'); 
    elseif l == 2
        save('Results/Data/Fig3_L','tau_vec','merger_share'); 
    elseif l == 3
        save('Results/Data/Fig3_inf','tau_vec','merger_share'); 
    else
        error('Not enough lambda values')
    end

end

sprintf('Convergence achieved');
