%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Maximilian Vogler                                               %
% Name: Inner_Loops_Tariff.m                                              %
% Date: 03/28/2021                                                        %
% Purpose: Solves the inner loops                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [KHH,KHF,KFH,KFF,LAMBDA_FH,LAMBDA_HF,PROF_HH,PROF_HF,SoverMU_FH,SoverMU_FF,TARIFF_FH,TOPF1,welfare_moments,IOTAH,PHM,SHM,checkmatH] = Inner_Loops_Tariff(sigma,theta,F,tau,ALPHA,RT,ZH,ZF,tariff,w,wF,Y0,YF0,vMU,BER,S,top)                      

tol = 1e-2;                                                                % Set tolerance level for A-B loop

%% Set up loops
MH=size(ZH,1);
MF=size(ZF,1);

% Find sectors who are subject to tariff
tariff_ind = tariff~=0;

% Rescale productivity of foreign by the tariff
ZF_tariff = ZF;

if top == 1 % only on top firm
    ZF_tariff(1,tariff_ind)=ZF(1,tariff_ind).*(1-tariff(tariff_ind));   
else % on all firms
    ZF_tariff(:,tariff_ind)=ZF(:,tariff_ind).*(1-ones(MF,1)*tariff(tariff_ind));
end

MCH=w./ZH;
MCF=wF./ZF;
MCF_tariff=wF./ZF_tariff;

F=F*476/S;
MCHM=[MCH;tau*MCF_tariff];
MCFM=[MCF;tau*MCH];

[MCHM,permH]=sort(MCHM,1);
[MCFM,permF]=sort(MCFM,1);                                                 % Sort marginal costs in home and foreign
IOTAH=(permH>MH);                                                          % Indicator whether firm is of foreign origin in home market
IOTAF=(permF>MF);                                                          % Indicator whether firm is of home origin in foreign market

MFMH=size(MCHM,1); 

%% Constant Markup
% Set home markups/shares/entry in CMU case
MKP0H=sigma/(sigma-1);                                                     % Markup
PHM=(MKP0H*MCHM).^(1-sigma);                                               % p_{i,z}^(1-sigma)                                            
SLASTH=PHM./cumsum(PHM);                                                   % Potential s_{i,z} (before entry decision)
checkmatH=(SLASTH>sigma/Y0*w*F./(repmat(ALPHA,MFMH,1)));                   % Indicator whether firm (i,z) enters in CMU case               
SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));                % Actual s_{i,z} (after entry decision)             
EPSH=sigma;                                                                % Used to compute markups at the end of the loop

% Same as above in foreign market
MKP0F=sigma/(sigma-1);
PFM=(MKP0F*MCFM).^(1-sigma);
SLASTF=PFM./cumsum(PFM);     
checkmatF=(SLASTF>sigma/YF0*wF*F./(repmat(ALPHA,MFMH,1)));                 
SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1));                  
EPSF=sigma;

%% Variable Markup 
% Enter loop in VMU case
if vMU==1
    EPSH=vMU_markup(sigma,SHM,BER);                                        % Compute variable markups with CMU market shares
    EPSH(EPSH<=1.25)=1.25;                                                                   
    
    EPSF=vMU_markup(sigma,SFM,BER);
    EPSF(EPSF<=1.25)=1.25; 
    
    iter=0;
    diff=1;
    while (diff>tol) && (iter<101)
        iter=iter+1;
        
        % Update shares in home market
        MKPNH=1/2*(MKP0H+EPSH./(EPSH-1));                                  % Set markups for current step
        PHM=(MKPNH.*MCHM).^(1-sigma);                                      % Set prices given markups           
        SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));        % Calculate shares given prices 
        
        EPSH=vMU_markup(sigma,SHM,BER);                                    % Recalculate markups given prices 
        EPSH(EPSH<=1.25)=1.25;                                             
        
        % Same in foreign market       
        MKPNF=1/2*(MKP0F+EPSF./(EPSF-1));
        PFM=(MKPNF.*MCFM).^(1-sigma);                                       
        SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1));  

        EPSF=vMU_markup(sigma,SFM,BER);
        EPSF(EPSF<=1.25)=1.25;
        
        % Check for convergence
        diff=sum(sum(abs([MKPNH-MKP0H;MKPNF-MKP0F])));
        
        % Update markups
        MKP0H=MKPNH;
        MKP0F=MKPNF;

    end
    
    % Check entry conditions again with variable markups
    PHM=(MKP0H.*MCHM).^(1-sigma);                                           
    SLASTH=PHM./cumsum(PHM);
    checkmatH=(SLASTH>sigma/Y0*w*F./(repmat(ALPHA,MFMH,1)));               % Determine which firms enter with CMU fringe - this is the crucial new element 
    SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));

    EPSH=vMU_markup(sigma,SHM,BER);
    EPSH(EPSH<=1.25)=1.25; 
    
    % Same in foreign market
    PFM=(MKP0F.*MCFM).^(1-sigma);                                          
    SLASTF=PFM./cumsum(PFM);     
    checkmatF=(SLASTF>sigma/YF0*wF*F./(repmat(ALPHA,MFMH,1)));              
    SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1)); 
    
    EPSF=vMU_markup(sigma,SFM,BER);
    EPSF(EPSF<=1.25)=1.25;
    
    % Same loop as before, but with new entrants
    iter=0;
    diff=1;
    while (diff>tol) && (iter<101)
        iter=iter+1;

        MKPNH=1/2*(MKP0H+EPSH./(EPSH-1));
        PHM=(MKPNH.*MCHM).^(1-sigma);                                     
        SHM=(PHM.*checkmatH)./(repmat(sum(PHM.*checkmatH),MFMH,1));
        
        EPSH=vMU_markup(sigma,SHM,BER);
        EPSH(EPSH<=1.25)=1.25;
        
        % Same in foreign market
        MKPNF=1/2*(MKP0F+EPSF./(EPSF-1));
        PFM=(MKPNF.*MCFM).^(1-sigma);                                           
        SFM=(PFM.*checkmatF)./(repmat(sum(PFM.*checkmatF),MFMH,1));   
        
        EPSF=vMU_markup(sigma,SFM,BER);
        EPSF(EPSF<=1.25)=1.25;

        % Check for convergence
        diff=sum(sum(abs([MKPNH-MKP0H;MKPNF-MKP0F])));

        % update markups
        MKP0H=MKPNH;
        MKP0F=MKPNF;

    end
end

checkmatH=(SHM>0);                                                         % Matrix of firms that enter in home in equilibrium above
checkmatF=(SFM>0); 
MUHM=EPSH./(EPSH-1);                                                       % Markups of firms in home
MUFM=EPSF./(EPSF-1);

%% Compute Summary Statistics 

% Auxiliary variables 
KHH = sum(checkmatH.*(1-IOTAH));                                           % Number of home firms active in home for each sector
KHF = sum(checkmatF.*IOTAF);                                               % Number of home firms active in foreign for each sector
KFH = sum(checkmatH.*(IOTAH));                                             % Number of foreign firms active in home for each sector
KFF = sum(checkmatF.*(1-IOTAF));                                           % Number of foreign firms active in foreign for each sector

LAMBDA_FH = sum(IOTAH.*SHM);
LAMBDA_HF = sum(IOTAF.*SFM);

PROF_HH = sum(SHM.*(1-IOTAH)./EPSH);
PROF_HF = sum(SFM.*IOTAF./EPSF);

SoverMU_FF = sum(SFM.*(1-IOTAF)./MUFM);

TOPF1 = max(IOTAH.*SHM);

if top == 1
    
    SoverMU_FH = sum(SHM.*IOTAH./MUHM) - max(SHM.*IOTAH./MUHM).*tariff;
    TARIFF_FH = TOPF1.*tariff;
    
else
    
    SoverMU_FH = sum(SHM.*IOTAH./MUHM).*(1-tariff);
    TARIFF_FH = LAMBDA_FH.*tariff;
    
end

% Welfare moments
P_index = sum(PHM.*checkmatH).^(1/(1-sigma));                              % Home price index
PF_index = sum(PFM.*checkmatF).^(1/(1-sigma));                             % Foreign price index
profits = sum(SHM.*(1-IOTAH)./EPSH).*ALPHA*Y0 + ...                        % Total profits of home firms (in both home and foreign markets) 
          sum(SFM.*IOTAF./EPSF).*ALPHA*YF0 - ...
          KHH*w*F-KHF*wF*F;    

welfare_moments = zeros(3,length(profits));
welfare_moments(1,:) = P_index;
welfare_moments(2,:) = profits;
welfare_moments(3,:) = PF_index;

end
