%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Maximilian Vogler                                               %
% Name: Main_Tariff.m                                                     %
% Date: 03/28/2021                                                        %
% Purpose: Create data for Fig6(a), Fig6(b), Fig7(a) and Fig7(b)          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Housekeeping
clear;
close all;
clc

tstart0 = tic;

addpath('Data');
addpath('Auxiliary_Functions');
addpath('Results');

%% Load Parameters 
load('Baseline_Parameters') 
load('GE_Results','Y','YF')

% Extract parameters
muT = bestParams(1,1);
sigmaT = bestParams(1,2);
tau = bestParams(1,3);
kappa = bestParams(1,4);
f = bestParams(1,5);

% Fix sigma
sigma = 5;

% Compute derivative parameters
theta = (sigma-1)*kappa;
F = f*4.93*.43*10^(-5)/5;

% Wages (with domestic wage as numeraire and calibrated relative wage)
wR = 1.13;      
w = 1;          
wF = 1/wR; 

% Labor Normalization
L0 = 100;

% VM/CM
vInd = 0;                                                                  % == 0 for constant markups,  1 for variable markups

%% Setup of model 

% Number of seeds over which median is taken
seed_num = 10;

% Number of bins
IP = 10;

plot_welfare_granular = zeros(seed_num,IP);
plot_welfarF_granular = zeros(seed_num,IP);
plot_revenue_granular = zeros(seed_num,IP);
plot_profits_granular = zeros(seed_num,IP);
plot_indices_granular = zeros(seed_num,IP);
plot_welfare_uniform = zeros(seed_num,IP);
plot_welfarF_uniform = zeros(seed_num,IP);
plot_revenue_uniform = zeros(seed_num,IP);
plot_profits_uniform = zeros(seed_num,IP);
plot_indices_uniform = zeros(seed_num,IP);

for s = 1:seed_num

% Initialize random number generator and set seed
aseed = s;
rng(aseed);

% Number of shadow firms
MF_large=10000;
MF_small=1400;
MH_large=5000;
MH_small=700;

% Markups
if vInd == 0
    vMU = 0;                                                               % = 1 for variable markups
elseif vInd == 1
    vMU = 1;
else
    error('vMu has to equal 0 or 1');
end

% Competition Structure
BER = 0;                                                                   % = 1 for Bertrand competition

% Compute number of sectors 
cdshares_init = csvread('cdshares_v3.csv');                                % Cobb-Douglas shares in data

S_multiple = 4;                                         
S_init = length(cdshares_init);                                            % Number of sectors in data
S = S_init*S_multiple;                                                     % Number of sectors used

% Assign CD-shares across sectors 
ALPHA = zeros(S,1);
ALPHA(1:S_init,1) = cdshares_init;                                         % Same as in the data
for i = 1:S_multiple - 1                                                   % Random permutations for additional sectors
    ALPHA(S_init*i+1:S_init*(i+1),1) = cdshares_init(randperm(S_init));
end
ALPHA = ALPHA/S_multiple;

% Split sectors into large and small based on CD-share.
split_param = 1.25;
small = (ALPHA<split_param/S);                            
Nsmall = sum(small);                           
Nlarge = S-Nsmall;

%% Productivity draws

rng(aseed);                                              
rtdraws = randn(1,S);

% Draw fundamental Productivites T_z for each z
RT=exp(muT+sigmaT*rtdraws);
RTS=RT(small);
RTL=RT(~small);

% Draw idiosyncratic productivities varphi_{i,z} for each z and i
% Small firm U_{i,z}
UH0S = exprnd(1,MH_small,Nsmall);                                          % Draw U of most productive small home shadow firm and spacings in each sector from exponential with mean 1
UF0S = exprnd(1,MF_small,Nsmall);                        
UHS = cumsum(UH0S);                                                        % Cumulate to get U of each home firm in each sector 
UFS = cumsum(UF0S);

% Large firm U_{i,z}
UH0L = exprnd(1,MH_large,Nlarge);
UF0L = exprnd(1,MF_large,Nlarge);
UHL = cumsum(UH0L);
UFL = cumsum(UF0L);

% Transform U_{i,z} into varphi_{i,z}  
ZHS=(UHS./(repmat(RTS,MH_small,1))).^(-1/theta);
ZFS=UFS.^(-1/theta);
ZHL=(UHL./(repmat(RTL,MH_large,1))).^(-1/theta);                        
ZFL=UFL.^(-1/theta);

%% Tariff parameters

uniform_tariff = 0.01;

%% Initialize matrices
bin_weight = zeros(IP,1);

Y_uniform = zeros(1,IP);
YF_uniform = zeros(1,IP);
Y_granular = zeros(1,IP);
YF_granular = zeros(1,IP);

uniform_local_revenue = zeros(S,IP);
uniform_local_profits = zeros(S,IP);
uniform_local_indices = zeros(S,IP);
uniform_local_indiceF = zeros(S,IP);

uniform_revenue = zeros(1,IP);
uniform_profits = zeros(1,IP);
uniform_indices = zeros(1,IP);
uniform_indiceF = zeros(1,IP);
uniform_welfare = zeros(1,IP);
uniform_welfarF = zeros(1,IP);

granular_local_revenue = zeros(S,IP);
granular_local_profits = zeros(S,IP);
granular_local_indices = zeros(S,IP);
granular_local_indiceF = zeros(S,IP);

granular_revenue = zeros(1,IP);
granular_profits = zeros(1,IP);
granular_indices = zeros(1,IP);
granular_indiceF = zeros(1,IP);
granular_welfare = zeros(1,IP);
granular_welfarF = zeros(1,IP);

passthrough = zeros(4,IP);
 
%% Compute GE baseline

% Guess home and foreign output Y and YF
Y0 = Y;
YF0 = YF;

% Run loops to solve GE model 
[Y_baseline,YF_baseline,LF,PHIVEC,LAMBDAHVEC,TOPF1,baseline_welfare_moments,baseline_cell] = GEreplication_vectorized(sigma,theta,F,tau,ALPHA,RTS,RTL,ZHS,ZFS,ZHL,ZFL,w,wF,L0,Y0,YF0,small,vMU,BER,1);

% Compute and store moments
relTOP1 = TOPF1./LAMBDAHVEC;
baseline_local_indices = baseline_welfare_moments(1,:)';
baseline_local_profits = baseline_welfare_moments(2,:)';
baseline_foreign_indices = baseline_welfare_moments(3,:)';

baseline_revenue = 0;
baseline_profits = sum(baseline_local_profits);
baseline_indices = exp(ALPHA'*log(baseline_local_indices));
baseline_indiceF = exp(ALPHA'*log(baseline_foreign_indices));
baseline_welfare = Y_baseline/baseline_indices;
baseline_welfarF = YF_baseline/baseline_indiceF;

disp('Baseline Computed')

% Compute granular trade share Gamma
Gamma = (LAMBDAHVEC - PHIVEC)';

[bounds,bin_index] = Binning(LAMBDAHVEC',IP);

%% Compute uniform tariffs

% Compute outcomes if tariffs are levied in different sector bins
for j = 1:IP
    
    disp('---------------------')
    disp('')
    
    % Compute weight of bin
    bin_weight(j) = sum(ALPHA(bin_index==j)); 

    top = 2;                                                               % Tariff applies to all foreign firms
    tariff = uniform_tariff * ones(1,S);
    tariff(bin_index~=j) = 0;   
    
    [Y_uniform(j),YF_uniform(j),wF,LAMBDAHVEC,~,uniform_welfare_moments,uniform_cell] = GE_Tariff(sigma,theta,F,tau,ALPHA,RTS,RTL,ZHS,ZHL,ZFS,ZFL,tariff,w,wF,L0,LF,Y0,YF0,small,vMU,BER,top);
    disp('Uniform Computed')
    disp('');
    uniform_tt = tariff;

    % Compute and store moments
    uniform_local_revenue(:,j) = (tariff.*LAMBDAHVEC)';
    uniform_local_indices(:,j) = uniform_welfare_moments(1,:)';
    uniform_local_profits(:,j) = uniform_welfare_moments(2,:)';
    uniform_local_indiceF(:,j) = uniform_welfare_moments(3,:)';
    
    uniform_revenue(j) = sum(uniform_local_revenue(:,j));
    uniform_profits(j) = sum(uniform_local_profits(:,j));
    uniform_indices(j) = exp(ALPHA'*log(uniform_local_indices(:,j)));
    uniform_indiceF(j) = exp(ALPHA'*log(uniform_local_indiceF(:,j)));
    uniform_welfare(j) = Y_uniform(j)/uniform_indices(j);
    uniform_welfarF(j) = YF_uniform(j)/uniform_indiceF(j);

%% Compute equivalent tariff on top 1 firm

    top = 1;
    tariff_guess = min(tariff./relTOP1, .75);
    tariff_guess(relTOP1==0)=0;      
    tariff_guess(isnan(relTOP1))=0;
    tariff_guess(bin_index~=j) = 0;
    diff_tariff = 1;
    innerLoop = 1;
    Y_granular(j) = Y_uniform(j);
    YF_granular(j) = YF_uniform(j);
    while (diff_tariff > 0.01) && (innerLoop < 40)
        
        [Y_granular(j),YF_granular(j),wF,~,TOPF1,granular_welfare_moments,granular_cell] = GE_Tariff(sigma,theta,F,tau,ALPHA,RTS,RTL,ZHS,ZHL,ZFS,ZFL,tariff_guess,w,wF,L0,LF,Y_granular(j),YF_granular(j),small,vMU,BER,top);
        
        granular_tt = tariff_guess;
        
        % Compute new revenues
        granular_local_revenue(:,j) = (tariff_guess.*TOPF1)';   
        
        % Compute new diff_tariff
        ratio_inter = granular_local_revenue(:,j)./uniform_local_revenue(:,j);   
        ratio = ratio_inter(bin_index==j);
        ratio(isnan(ratio)) = 1;
        diff_tariff = max(abs(ratio(ratio>0)-1));
        disp('Find tariff number:')
        disp(innerLoop)
        disp(diff_tariff)
        
        % Update tariff guess
        tariff_new = tariff_guess;
        tariff_new(bin_index'==j) = tariff_guess(bin_index'==j)./ratio'.^(1/2);
        tariff_new(tariff_guess==0) = 0;
        tariff_new(isnan(tariff_guess)) = 0;
        tariff_guess = tariff_new;  
   
        innerLoop = innerLoop + 1;
        
    end
    
    disp('Granular Computed')
    disp('');
    
    % Compute and store moments
    granular_local_indices(:,j) = granular_welfare_moments(1,:)'; 
    granular_local_profits(:,j) = granular_welfare_moments(2,:)';
    granular_local_indiceF(:,j) = granular_welfare_moments(3,:)';
    
    granular_revenue(j) = sum(granular_local_revenue(:,j));
    granular_profits(j) = sum(granular_local_profits(:,j));
    granular_indices(j) = exp(ALPHA'*log(granular_local_indices(:,j)));
    granular_indiceF(j) = exp(ALPHA'*log(granular_local_indiceF(:,j)));
    granular_welfare(j) = Y_granular(j)/granular_indices(j);
    granular_welfarF(j) = YF_granular(j)/granular_indiceF(j);
    
    %% Passthrough moments
    
    % Identify treatment sectors
    
    treatment = (bin_index==j)';
    treatmentS = treatment(small);
    treatmentL = treatment(~small);
    
    ALPHAS = ALPHA(small);
    ALPHAL = ALPHA(~small);
    ALPHAS = ALPHAS(treatmentS);
    ALPHAL = ALPHAL(treatmentL);
    
    granular_ttS = granular_tt(small);
    granular_ttL = granular_tt(~small);
    granular_ttS = granular_ttS(treatmentS);
    granular_ttL = granular_ttL(treatmentL);
    
    % Unpack data
    PHMS_baseline = baseline_cell{1,1}(:,treatmentS);
    SHMS_baseline = baseline_cell{2,1}(:,treatmentS);
    
    PHML_baseline = baseline_cell{1,2}(:,treatmentL);
    SHML_baseline = baseline_cell{2,2}(:,treatmentL);
    
    IOTAHS_uniform = uniform_cell{1,1}(:,treatmentS);
    PHMS_uniform = uniform_cell{2,1}(:,treatmentS);
    SHMS_uniform = uniform_cell{3,1}(:,treatmentS);
    checkmatHS_uniform = uniform_cell{4,1}(:,treatmentS);
    
    IOTAHL_uniform = uniform_cell{1,2}(:,treatmentL);
    PHML_uniform = uniform_cell{2,2}(:,treatmentL);
    SHML_uniform = uniform_cell{3,2}(:,treatmentL);
    checkmatHL_uniform = uniform_cell{4,2}(:,treatmentL);
    
    IOTAHS_granular = granular_cell{1,1}(:,treatmentS);
    PHMS_granular = granular_cell{2,1}(:,treatmentS);
    SHMS_granular = granular_cell{3,1}(:,treatmentS);
    checkmatHS_granular = granular_cell{4,1}(:,treatmentS);
    
    IOTAHL_granular = granular_cell{1,2}(:,treatmentL);
    PHML_granular = granular_cell{2,2}(:,treatmentL);
    SHML_granular = granular_cell{3,2}(:,treatmentL);
    checkmatHL_granular = granular_cell{4,2}(:,treatmentL);
    
    % Identify largest foreign firm in treatment sectors
    [maxS_uniform,argmaxS_uniform] = max(IOTAHS_uniform.*SHMS_uniform);
    [maxL_uniform,argmaxL_uniform] = max(IOTAHL_uniform.*SHML_uniform);
    
    [maxS_granular,argmaxS_granular] = max(IOTAHS_granular.*SHMS_granular);
    [maxL_granular,argmaxL_granular] = max(IOTAHL_granular.*SHML_granular);
    
    
    mom1 = 0;
    for i = 1:sum(treatmentS)
        x = (log(PHMS_uniform(argmaxS_uniform(i),i)) - log(PHMS_baseline(argmaxS_uniform(i),i)))/(log(1+uniform_tariff))*ALPHAS(i)/(sum(ALPHAS)+sum(ALPHAL));
        if ~isnan(x)
            mom1 = mom1 + x;
        end
    end
    for i = 1:sum(treatmentL)
        x = (log(PHML_uniform(argmaxL_uniform(i),i)) - log(PHML_baseline(argmaxL_uniform(i),i)))/log(1+uniform_tariff)*ALPHAL(i)/(sum(ALPHAS)+sum(ALPHAL));
        if ~isnan(x)
            mom1 = mom1 + x;
        end
    end
    
    mom2 = 0;
    for i = 1:sum(treatmentS)
        x = (log(PHMS_granular(argmaxS_granular(i),i)) - log(PHMS_baseline(argmaxS_granular(i),i)))/log(1+granular_ttS(i))*ALPHAS(i)/(sum(ALPHAS)+sum(ALPHAL));
        if ~isnan(x) && ~isinf(x) 
            mom2 = mom2 + x;
        end
    end
    for i = 1:sum(treatmentL)
        x = (log(PHML_granular(argmaxL_granular(i),i)) - log(PHML_baseline(argmaxL_granular(i),i)))/log(1+granular_ttL(i))*ALPHAL(i)/(sum(ALPHAS)+sum(ALPHAL));
        if ~isnan(x) && ~isinf(x)
            mom2 = mom2 + x;
        end
    end
    
    indexS_uniform = IOTAHS_uniform.*checkmatHS_uniform>0;
    indexL_uniform = IOTAHL_uniform.*checkmatHL_uniform>0;
    
    indexS_granular = IOTAHS_granular.*checkmatHS_granular>0;
    indexL_granular = IOTAHL_granular.*checkmatHL_granular>0;
    
    mom3 = 0;
    for i = 1:sum(treatmentS)
        x = sum((log(PHMS_uniform(indexS_uniform(:,i),i)) - log(PHMS_baseline(indexS_uniform(:,i),i))).*SHMS_baseline(indexS_uniform(:,i),i)).*ALPHAS(i)/(sum(ALPHAS)+sum(ALPHAL))/sum(SHMS_baseline(indexS_uniform(:,i),i));
        if ~isnan(x)
           mom3 = mom3 + x;
        end
    end
    for i = 1:sum(treatmentL)
        x = sum((log(PHML_uniform(indexL_uniform(:,i),i)) - log(PHML_baseline(indexL_uniform(:,i),i))).*SHML_baseline(indexL_uniform(:,i),i)).*ALPHAL(i)/(sum(ALPHAS)+sum(ALPHAL))/sum(SHML_baseline(indexL_uniform(:,i),i));
        if ~isnan(x)
            mom3 = mom3 + x; 
        end
    end
    mom3 = mom3 / log(1+uniform_tariff);
    
    mom4 = 0;
    for i = 1:sum(treatmentS)
        x = sum((log(PHMS_granular(indexS_granular(:,i),i)) - log(PHMS_baseline(indexS_granular(:,i),i))).*SHMS_baseline(indexS_granular(:,i),i)).*ALPHAS(i)/(sum(ALPHAS)+sum(ALPHAL))/sum(SHMS_baseline(indexS_granular(:,i),i));
        if ~isnan(x)
            mom4 = mom4 + x;
        end
    end
    for i = 1:sum(treatmentL)
        x = sum((log(PHML_granular(indexL_granular(:,i),i)) - log(PHML_baseline(indexL_granular(:,i),i))).*SHML_baseline(indexL_granular(:,i),i)).*ALPHAL(i)/(sum(ALPHAS)+sum(ALPHAL))/sum(SHML_baseline(indexL_granular(:,i),i));
        if ~isnan(x)
            mom4 = mom4 + x;
        end
    end
    mom4 = mom4 / log(1+uniform_tariff);
    
    passthrough(1,j) = mom1;
    passthrough(2,j) = mom2;
    passthrough(3,j) = mom3;
    passthrough(4,j) = mom4;
    
    fprintf('Bin %d/%d of seed %d/%d has been computed.\n', j, IP, s, seed_num)
    disp('---------------------')
    
end

%% Plots

% Plot Welfare Decomposition for Uniform Tariff

plot_welfare_uniform(s,:) = (uniform_welfare./baseline_welfare-1)*100; 
plot_welfarF_uniform(s,:) = (uniform_welfarF./baseline_welfarF-1)*100; % Note that all of these are vectors over IPs
plot_revenue_uniform(s,:) = sum(ALPHA.*uniform_local_revenue.*Y_uniform)/Y_baseline*100;
plot_profits_uniform(s,:) = (uniform_profits-baseline_profits)/Y_baseline*100;
plot_indices_uniform(s,:) = sum(ALPHA.*(log(uniform_local_indices)-log(baseline_local_indices)))*100; 

% Plot Welfare Decomposition for Granular Tariff

plot_welfare_granular(s,:) = (granular_welfare./baseline_welfare-1)*100;
plot_welfarF_granular(s,:) = (granular_welfarF./baseline_welfarF-1)*100; % Note that all of these are vectors over IPs
plot_revenue_granular(s,:) = sum(ALPHA.*granular_local_revenue.*Y_granular)/Y_baseline*100;
plot_profits_granular(s,:) = (granular_profits-baseline_profits)/Y_baseline*100;
plot_indices_granular(s,:) = sum(ALPHA.*(log(granular_local_indices)-log(baseline_local_indices)))*100;

fprintf('Seed %d/%d has been computed.\n', s, seed_num)
disp('');

end

if vInd == 0
    save('Results/Data/Data_CM','plot_welfare_uniform','plot_welfarF_uniform','plot_revenue_uniform','plot_profits_uniform','plot_indices_uniform',...
    'plot_welfare_granular','plot_welfarF_granular','plot_revenue_granular','plot_profits_granular','plot_indices_granular');
else 
    save('Results/Data/Data_VM','plot_welfare_uniform','plot_welfarF_uniform','plot_revenue_uniform','plot_profits_uniform','plot_indices_uniform',...
    'plot_welfare_granular','plot_welfarF_granular','plot_revenue_granular','plot_profits_granular','plot_indices_granular');
end
